#!/bin/bash
# Runs every few minutes and processes each active client and adds next
# doc or random doc to queue for the client.

Now() {
	date '+%s'
}

Date() {
	NOW=$(Now)
	NOW=$(($NOW + ($1 * 86400) + ($RANDOM * $2)))
	date --date="@$NOW" '+%d/%m/%Y'
}

Time() {
	date '+%H:%M:%S'
}

GenerateJob() {
	DEVICE=$1
	ID=$2
	FIELDS=`awk '/^FIELDS:/ { print substr($0,8); exit 0; }' docs/job.data`
	COUNT=`awk '/^COUNT:/ { print $2; exit 0; }' docs/job.data`
	if test $ID -gt $COUNT ; then
		echo donejobs $(Now) >spool/$DEVICE
		return
	fi
	DATA=`awk '/^DATA:'$ID'\|/ { print substr($0,6); exit 0; }' docs/job.data`
	IFS=\| ; set -- $DATA ; IFS=" 	"
	N=0
	SED=""
	for field in $FIELDS ; do
		N=$(($N+1))
		eval "V=\${$N}"
		eval "V=\"$V\""
		eval "$field=\"$V\""
		SED="${SED}s|{{$field}}|$V|g;"
	done
	sed "$SED" docs/job.template > docs/job${ID}.rmc
	echo "$DEVICE: Send job $ID"
	echo job$ID.rmc >>queue_${DEVICE}.txt
	echo job$ID >spool/$DEVICE
	if test -s token_${DEVICE}.txt ; then
		echo "$DEVICE: send notification for job $ID"
		./push $DEVICE notify "New Job $ID" "New priority 0 job ticket is available!" p0 ring1.mp3 100,10 sync job$ID | tee /tmp/push$$
		if grep ERROR /tmp/push$$ >/dev/null ; then
			echo "$DEVICE: status=ERROR"
			# Remove device
			rm queue_${DEVICE}*.txt token_${DEVICE}.txt
			echo "$DEVICE: removed"
		else
			echo "$DEVICE: status=SUCCESS"
		fi
		rm /tmp/push$$
	fi
}

Cleanup() {
	echo "$1: cleanup... (TODO)"
	rm -f queue_${1}.txt queue_${1}_sent.txt token_${1}.txt spool/$1
	rm -f IN/${1}-*
}

ProcessState() {
	case "$2" in
	new) GenerateJob $1 1 ;;
	job*) 
		JOB=$((${2#job}+1))
		GenerateJob $1 $JOB
		;;
	donejobs)
		read donejobs when < spool/$1
		echo "$1: when=$when"
		case "$when" in
		"") ;;
		*)	AGE=$(($(Now) - $when))
			echo "$1: completed $AGE seconds ago"
			if test $AGE -gt 86400 ; then
				Cleanup $1
			fi
			;;
		esac
		;;
	esac
}

Process() {
	echo "$1: Process ..."
	if test -f spool/$1 ; then
		state=$(cat spool/$1)
	else
		state=new
	fi
	ProcessState $1 $state
}

echo
echo "$(date) demo.sh started"

cd /var/www-sites/rmc/server

SPOOLS=$(ls spool 2>/dev/null)
for DEVICE in $SPOOLS
do
	if test ! -f queue_$DEVICE.txt ; then
		echo "$DEVICE: clean up old spool file"
		rm -f spool/$DEVICE
	fi
done

DEVICES=$(ls queue_*.txt 2>/dev/null | sed "s/queue_//g;s/.txt//g")
for DEVICE in $DEVICES
do
	case "$DEVICE" in
	*_sent) ;;
	*) Process $DEVICE ;;
	esac
done
echo "$(date) demo.sh finished"
